
local addon = KiwiPlates

--=================================================================
-- Player CastBar Skining
--=================================================================

local castBar = CastingBarFrame -- blizzard player casting bar

addon.defaults.playerCastBar = {}

local function UpdateCastBar()
	local db = addon.db.playerCastBar
	local w = db.barWidth or 300
	local h = db.barHeight or 20
	castBar:ClearAllPoints()
	castBar:SetPoint("BOTTOM", UIParent, "BOTTOM", (db.offsetX or 0) + (db.iconEnabled and h/2 or 0), db.offsetY or 55)
	castBar:SetSize( w, h )
	castBar:SetStatusBarTexture( addon.TexCache[db.barTexture] )
	castBar.Text:ClearAllPoints()
	castBar.Text:SetAllPoints()
	castBar.Text:SetFont( addon.FontCache[db.barFontFile or 'Roboto Condensed Bold'], db.barFontSize or 12, db.barFontFlags or 'OUTLINE' )
	castBar.Icon:ClearAllPoints()
	castBar.Icon:SetPoint("RIGHT", castBar, "LEFT", -1, 0)
	castBar.Icon:SetSize( h, h )
	castBar.Icon:SetShown(db.iconEnabled)
	castBar.Border:ClearAllPoints()
	castBar.Border:SetPoint("TOPLEFT", castBar, "TOPLEFT", -1, 1)
	castBar.Border:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", 1, -1)
	castBar.Border:SetDrawLayer("BACKGROUND", 0)
	castBar.Border:SetColorTexture(0,0,0,1)
	castBar.Flash:SetColorTexture(0,0,0,0)
	for i = castBar:GetNumRegions(),1,-1 do
		local tex = select(i, castBar:GetRegions())
		if tex and tex:IsObjectType('Texture') and tex:GetDrawLayer()=='BACKGROUND' and tex~=castBar.Border then
			tex:SetDrawLayer('BACKGROUND',1)
			tex:SetColorTexture(.1,.1,.1,1)
			break
		end
	end
end

-- Init

CastingBarFrame.ignoreFramePositionManager = true
addon:RegisterMessage('ENABLE', function()
	if addon.db.playerCastBar.enabled == true then
		UpdateCastBar()
	elseif addon.db.playerCastBar.enabled == false then
		CastingBarFrame.RegisterEvent = function() end
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:Hide()
	end
end )

-- Options

local function Update()
	if addon.db.playerCastBar.enabled == true then
		UpdateCastBar()
		castBar.casting = true
		castBar.maxValue = 500
		castBar.value = castBar.maxValue / 3
		castBar:SetMinMaxValues(0, castBar.maxValue)
		castBar.Text:SetText('Spell Name Example')
		castBar.Icon:SetTexture( 'Interface\\ICONS\\Ability_Creature_Cursed_05' )
		castBar.Flash:Hide()
		castBar:Show()
		CastingBarFrame_ApplyAlpha(castBar, 1)
	end
end

addon:SetupOptions( 'Extras', 'CastBar(Player)', {
	barEnabled = {
		type = 'select',
		order = 0,
		width = 'normal',
		name = 'Select Bar Type:',
		get = function()
			local v = addon.db.playerCastBar.enabled
			return (v==nil and 1) or (v==true and 2) or 3
		end,
		set = function(info, v)
			if v==1 then
				addon.db.playerCastBar.enabled = nil
			else
				addon.db.playerCastBar.enabled = (v==2)
			end
			ReloadUI()
		end,
		values = { addon.FormatTitle('Blizzard Casting Bar'), addon.FormatTitle('Kiwi Casting Bar'), addon.FormatTitle('Disable Casting Bar') },
		confirm = function() return "An UI Reload is required to change this option. Are you sure ?" end,
	},
	header1 = { type = "header", order = 5, name = "Casting Bar for Player" },
	barOffsetX =  {
		type = 'range', order = 6, width = "full", name = 'X Offset', softMin = -500, softMax = 500, step = 1,
		get = function() return addon.db.playerCastBar.offsetX or 0 end,
		set = function(info,value)
			addon.db.playerCastBar.offsetX = value
			Update()
		end,
	},
	barOffsetY =  {
		type = 'range', order = 7, width = "full", name = 'Y Offset', softMin = 0, softMax = math.floor(GetScreenHeight()+.5), step = 1,
		get = function() return addon.db.playerCastBar.offsetY or 55 end,
		set = function(info,value)
			addon.db.playerCastBar.offsetY = value
			Update()
		end,
	},
	barWidth = {
		type = 'range', order = 11, width = "full", name = 'Bar Width', min = 0, softMax = 500, step = 1,
		get = function() return addon.db.playerCastBar.barWidth	or 200 end,
		set = function(info,value)
			addon.db.playerCastBar.barWidth = value
			Update()
		end,
	},
	barHeight =  {
		type = 'range', order = 12, width = "full", name = 'Bar Height', min = 1, softMax = 64, step = 1,
		get = function() return addon.db.playerCastBar.barHeight or 16 end,
		set = function(info,value)
			addon.db.playerCastBar.barHeight = value
			Update()
		end,
	},
	barTexture = {
		type = "select", dialogControl = "LSM30_Statusbar",
		order = 13,
		name = "Bar Texture",
		desc = "Adjust the bar texture.",
		get = function (info) return addon.db.playerCastBar.barTexture or "Minimalist" end,
		set = function (info, v)
			addon.db.playerCastBar.barTexture = v
			Update()
		end,
		values = AceGUIWidgetLSMlists.statusbar,
	},
	barShowIcon = {
		type = "toggle",
		order = 14,
		name = "Show Spell Icon",
		get = function() return addon.db.playerCastBar.iconEnabled end,
		set = function (_, value)
			addon.db.playerCastBar.iconEnabled = value or nil
			Update()
		end,
	},
	header2 = { type = "header", order = 20, name = "Spell Text" },
	cbFontFile = {
		type = "select", dialogControl = "LSM30_Font",
		order = 21,
		name = "Font Name",
		values = AceGUIWidgetLSMlists.font,
		get = function () return addon.db.playerCastBar.barFontFile or 'Roboto Condensed Bold' end,
		set = function (_, v)
			addon.db.playerCastBar.barFontFile = v
			Update()
		end,
	},
	cbFontFlags = {
		type = "select",
		order = 22,
		name = "Font Border",
		values = addon.fontFlagsValues,
		get = function () return addon.db.playerCastBar.barFontFlags or "OUTLINE" end,
		set = function (_, v)
			addon.db.playerCastBar.barFontFlags =  v ~= "OUTLINE" and v or nil
			Update()
		end,
	},
	cbFontSize = {
		type = "range",
		order = 23,
		name = 'Font Size',
		min = 1,
		softMax =50,
		step = 1,
		get = function () return addon.db.playerCastBar.barFontSize or 8 end,
		set = function (_, v)
			addon.db.playerCastBar.barFontSize = v~=8 and v or nil
			Update()
		end,
	},
	testmode = {
		type = 'execute',
		order = 100,
		width = 'full',
		name = "Toggle Casting Bar Visibility",
		desc = 'Show/Hide the casting bar.',
		func = function()
			if castBar and castBar:IsVisible() then
				castBar:Hide()
			else
				Update()
			end
		end,
		hidden = function() return not addon.db.playerCastBar.enabled end,
	}

}, nil, { disabled = function() return addon.db.playerCastBar.enabled~=true end } )

