
local addon = KiwiPlates

local CastingBarFrame_SetUnit = KiwiPlatesCastingBarFrame_SetUnit

--=================================================================
-- Target CastBar Implementation
--=================================================================

local targetCastBar

addon.defaults.targetCastBar = {}

local function UpdatePlayerTarget()
	CastingBarFrame_SetUnit(targetCastBar, 'target')
end

local function UpdateTargetCastBar()
	local db = addon.db.targetCastBar
	if db.enabled then
		LibStub("LibKiwiClassicCast-1.0"):Register(targetCastBar)
		targetCastBar = targetCastBar or CreateFrame("StatusBar", nil, UIParent, "KiwiPlatesCastingBarFrameTemplate")
		targetCastBar:Hide()
		targetCastBar:ClearAllPoints()
		targetCastBar:SetPoint("CENTER", UIParent, "CENTER", (db.offsetX or 0)+(db.barHeight or 20)/2, db.offsetY or 250)
		targetCastBar:SetSize( db.barWidth or 300, db.barHeight or 20)
		targetCastBar.Icon:SetSize( targetCastBar:GetHeight(), targetCastBar:GetHeight() )
		targetCastBar.Text:SetFont( addon.FontCache[db.barFontFile or 'Roboto Condensed Bold'], db.barFontSize or 12, db.barFontFlags or 'OUTLINE' )
		targetCastBar:SetStatusBarTexture( addon.TexCache[db.barTexture] )
		targetCastBar.Icon:Show()
		targetCastBar:SetScript('OnEvent', UpdatePlayerTarget)
		targetCastBar:RegisterEvent("PLAYER_TARGET_CHANGED")
		UpdatePlayerTarget()
		addon.targetCastBar = targetCastBar
	elseif targetCastBar then
		LibStub("LibKiwiClassicCast-1.0"):Unregister(targetCastBar)
		targetCastBar:SetScript('OnEvent', nil)
		targetCastBar:UnregisterEvent("PLAYER_TARGET_CHANGED")
		CastingBarFrame_SetUnit(targetCastBar, nil)
		targetCastBar:Hide()
	end
end

-- Initialization

addon:RegisterMessage('UPDATE', UpdateTargetCastBar )

-- Options

local function Update()
	UpdateTargetCastBar()
	if addon.db.targetCastBar.enabled then
		targetCastBar.maxValue = 500
		targetCastBar.value = targetCastBar.maxValue / 3
		targetCastBar:SetMinMaxValues(0, targetCastBar.maxValue)
		targetCastBar.Text:SetText('Spell Name Example')
		targetCastBar.Icon:SetTexture( 'Interface\\ICONS\\Ability_Creature_Cursed_05' )
		targetCastBar:Show()
	end
end

-- addon.OptionsTable.args.Extras = { type = "group", order = 25, name = 'Extras', childGroups = "tab", args = {} }

addon:SetupOptions( 'Extras', 'CastBar(Target)', {
	barEnabled = {
		type = "toggle",
		order = 0, width = "double",
		name = addon.FormatTitle("Casting Bar for Target", true),
		get = function() return addon.db.targetCastBar.enabled end,
		set = function (_, value)
			addon.db.targetCastBar.enabled = value or nil
			Update()
		end,
	},
	header1 = { type = "header", order = 5, name = "Cast Bar" },
	barOffsetX =  {
		type = 'range', order = 6, width = "full", name = 'X Offset', softMin = -500, softMax = 500, step = 1,
		get = function() return addon.db.targetCastBar.offsetX or 0 end,
		set = function(info,value)
			addon.db.targetCastBar.offsetX = value
			Update()
		end,
	},
	barOffsetY =  {
		type = 'range', order = 7, width = "full", name = 'Y Offset', softMin = -500, softMax = 500, step = 1,
		get = function() return addon.db.targetCastBar.offsetY or 250 end,
		set = function(info,value)
			addon.db.targetCastBar.offsetY = value
			Update()
		end,
	},
	barWidth = {
		type = 'range', order = 11, width = "full", name = 'Bar Width', min = 0, softMax = 500, step = 1,
		get = function() return addon.db.targetCastBar.barWidth	or 200 end,
		set = function(info,value)
			addon.db.targetCastBar.barWidth = value
			Update()
		end,
	},
	barHeight =  {
		type = 'range', order = 12, width = "full", name = 'Bar Height', min = 1, softMax = 64, step = 1,
		get = function() return addon.db.targetCastBar.barHeight or 16 end,
		set = function(info,value)
			addon.db.targetCastBar.barHeight = value
			Update()
		end,
	},
	barTexture = {
		type = "select", dialogControl = "LSM30_Statusbar",
		order = 13,
		name = "Bar Texture",
		desc = "Adjust the bar texture.",
		get = function (info) return addon.db.targetCastBar.barTexture or "Minimalist" end,
		set = function (info, v)
			addon.db.targetCastBar.barTexture = v
			Update()
		end,
		values = AceGUIWidgetLSMlists.statusbar,
	},
	header2 = { type = "header", order = 20, name = "Spell Text" },
	cbFontFile = {
		type = "select", dialogControl = "LSM30_Font",
		order = 21,
		name = "Font Name",
		values = AceGUIWidgetLSMlists.font,
		get = function () return addon.db.targetCastBar.barFontFile or 'Roboto Condensed Bold' end,
		set = function (_, v)
			addon.db.targetCastBar.barFontFile = v
			Update()
		end,
	},
	cbFontFlags = {
		type = "select",
		order = 22,
		name = "Font Border",
		values = addon.fontFlagsValues,
		get = function () return addon.db.targetCastBar.barFontFlags or "OUTLINE" end,
		set = function (_, v)
			addon.db.targetCastBar.barFontFlags =  v ~= "OUTLINE" and v or nil
			Update()
		end,
	},
	cbFontSize = {
		type = "range",
		order = 23,
		name = 'Font Size',
		min = 1,
		softMax =50,
		step = 1,
		get = function () return addon.db.targetCastBar.barFontSize or 8 end,
		set = function (_, v)
			addon.db.targetCastBar.barFontSize = v~=8 and v or nil
			Update()
		end,
	},
	testmode = {
		type = 'execute',
		order = 100,
		width = 'full',
		name = "Toggle Casting Bar Visibility",
		desc = 'Show or Hide the casting bar.',
		func = function()
			if targetCastBar and targetCastBar:IsVisible() then
				targetCastBar:Hide()
			else
				Update()
			end
		end,
		hidden = function() return not addon.db.targetCastBar.enabled end,
	}

}, nil, { disabled = function() return not addon.db.targetCastBar.enabled end } )
